#!/bin/sh

#halt on all errors
set -e

#check for install-help option to print some help
if [ "$1" == "installhelp" ] || [ "$1" == "help" ] || [ "$1" == "--help" ] || [ "$1" == "-h" ]
then
  echo "This script creates hedes Image for the ADSL Router U.S.Robotics USR9108"
  echo "Go to http://www.usr.com/ and download $USRVERSION"
  echo "(search for GPL Code Central)"
  echo "untar this file and run the consumer_install script"
  echo "this needs the rpm (RedHat Package Manager) tool"
  echo "for Distributions like Ubuntu, Debian or Gentoo, which are not rpm-based"
  echo " you have to install rpm and edit the consumer_install script to run \"rpm --nodeps\""
  echo " check if you have gcc, libstdc++5 ncurses (incl. *-dev) and python installed"
  echo " finaly run this script (there should also be a README in this distribution)"
  echo "... have fun :-) and send some feedback "
  echo -e "\n     hedeimage@funforce.de (german or english)"
  exit 1
fi

HPATCHDIR=`pwd`

#check if we are in our directory
 if [ ! -f "$HPATCHDIR/create-hedeimage.sh" ]
 then
   cd "`dirname "$0"`"
   HPATCHDIR=`pwd`
 fi

#check all md5sums
 if [ ! -f md5sums.txt ]
 then
   echo "md5sums.txt is missing, this hedeimage-distribution is incomplete"
 fi
 set +e
 md5sum -c md5sums.txt
 if [ $? != "0" ]
 then
   echo "at least one md5sum is incorrect -> program abort"
   echo "either the distribution is broken or you have edited at least one file"
   echo " edit md5sums.txt to your needs: rm md5sums.txt; md5sum * > md5sums.txt"
   exit 1
 fi
 set -e

MFVERMD5HEDE="f39025c7bb1083fa70610de157a95404  /opt/USR910x_router/Makefile"
VERMD5HEDE="7325e24c64203a83dbd6bb422515a05f  /opt/USR910x_router/version.make"
MFVERMD5CURR=`md5sum /opt/USR910x_router/Makefile` 
VERMD5CURR=`md5sum /opt/USR910x_router/version.make` 
USRVERSION="USR9108_release1.3.tar.gz"

#check if USR9108-sources are installed and ready to use
if [ "$MFVERMD5HEDE" != "$MFVERMD5CURR" ] || [ "$VERMD5HEDE" != "$VERMD5CURR" ]
then
  if [ "$VERMD5HEDE" = "$VERMD5CURR" ]
  then
    echo "Seems there is some source in /opt/USR910x_router, but it's not usable."
    echo "Maybe you have patched it already..."
    echo "If you have tried to patch and something has gone wrong, "
    echo "  delete it and reinstall original USR sources"
    echo "Not patching this source!"
    exit 1
  fi
  if [ -f /opt/USR910x_router/version.make ]
  then
    echo "Seems there is some source in /opt/USR910x_router, but it's not usable."
    echo "Seems to be the wrong Version. This hede-Image needs:"
    echo " Original USRobotics Sources for USR9108: $USRVERSION"
    echo " installed to /opt/toolchain and /opt/USR910x_router"
    echo "Not patching this source!"
    exit 1
  fi
  echo "Seems there is no source in /opt/USR910x_router."
  echo "expand USR9108-source to /opt/USR910x_router/"
  echo "run this script with "installhelp"-option for some help on doing this"
  echo "Nothing to patch!"
  exit 1
fi  

#patch mconf.c for gcc4
 cd $HPATCHDIR
 GCCVER=`gcc --version | grep gcc | sed -s "s/^gcc.*\(.\)\..\...*$/\1/"`
 if [ $GCCVER -ge 4 ]
 then
   patch -d /opt/USR910x_router/kernel/linux/scripts/kconfig/ < mconf.c-gcc4.patch 
 fi

#patch Makefile (USR910x_router)
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/ < Makefile.patch

#patch targetfiles
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/targets/96348GWB < targetBfile.patch
 patch -d /opt/USR910x_router/targets/96348GW < targetAfile.patch

#patch kernel-config (for usb-storage etc)
#kernel config gets overwritten by devconfig-bcm.template
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/hostTools/scripts/ < defconfig-bcm.template.patch
 patch -d /opt/USR910x_router/hostTools/scripts/ < gendefconfig.patch

#usb-LED-blinking 
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/kernel/linux/drivers/usb/core/ < urb.c.patch

#patch busybox-config (for some helpful tools like "ls" e.g.)
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/userapps/opensource/busybox/ < busybox-brcm.config.patch
 patch -d /opt/USR910x_router/userapps/opensource/busybox/include < busybox-applets.patch
 patch -d /opt/USR910x_router/userapps/opensource/busybox/include < busybox-usage.patch
 patch -d /opt/USR910x_router/userapps/opensource/busybox/util-linux < busybox-ledctrl.patch
 patch -d /opt/USR910x_router/userapps/opensource/busybox/miscutils < busybox-crond.patch
 cp /opt/USR910x_router/userapps/opensource/busybox/ledctrl.c ../USR910x_router/userapps/opensource/busybox/util-linux/


#patch filesystem
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/targets < makeDevs.patch

#patch initsystem
 patch -d /opt/USR910x_router/targets < buildFS.patch 
 cp rcS.hede /opt/USR910x_router/targets/fs.src/etc/init.d/
 chmod +x /opt/USR910x_router/targets/fs.src/etc/init.d/rcS.hede
 echo -e "\n\n" >> /opt/USR910x_router/targets/fs.src/etc/init.d/rcS
 echo /etc/init.d/rcS.hede >> /opt/USR910x_router/targets/fs.src/etc/init.d/rcS

#expand tcp-wrapper, portmap and nfs
#patched Makefiles (tcp-wrapper, portmap and nfs) for hede-Image are included
 cd /opt/USR910x_router/userapps/opensource/
 tar xfz $HPATCHDIR/nfs-progs.tar.gz
 mkdir -p /opt/USR910x_router/targets/fs.src/mnt/
 cp $HPATCHDIR/NFS-mount-info.txt /opt/USR910x_router/targets/fs.src/mnt/README.txt
 mkdir -p /opt/USR910x_router/targets/fs.src/usr/sbin
 cp $HPATCHDIR/makevarlibnfsfiles /opt/USR910x_router/targets/fs.src/usr/sbin/
 cp $HPATCHDIR/exports.fix /opt/USR910x_router/targets/fs.src/etc/
 chmod +x /opt/USR910x_router/targets/fs.src/usr/sbin/makevarlibnfsfiles

#expand ez-ipupdate
#patched Makefile (ez-ipupdate) for hede-Image is included
 cd /opt/USR910x_router/userapps/opensource/
 tar xfz $HPATCHDIR/ez-ipupdate.tar.gz

#patch /etc/ppp
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/targets/fs.src/etc/ppp/ < ppp.patch
 mkdir -p /opt/USR910x_router/targets/fs.src/usr/bin/
 cp antidieb /opt/USR910x_router/targets/fs.src/usr/bin/antidieb
 patch -d /opt/USR910x_router/userapps/opensource/ppp/pppoe/ < pppoe.patch

#patch ftpd to not only be a firmware flasher
# with this it's (maybe?) no longer possible to flash firmware using ftp
 cd $HPATCHDIR
 patch -d /opt/USR910x_router/userapps/opensource/ftpd < ftpd.patch

#patch udhcpd to run /var/dhcpSentAck.sh every DHCPACK
# with this it's possible to change HDD state on Client powerup
 cd $HPATCHDIR
 mkdir -p /opt/USR910x_router/targets/fs.src/lib/
 cp dhcpSentAck.sh /opt/USR910x_router/targets/fs.src/lib/
 chmod +x /opt/USR910x_router/targets/fs.src/lib/dhcpSentAck.sh
 cp arpinger.sh /opt/USR910x_router/targets/fs.src/lib/
 chmod +x /opt/USR910x_router/targets/fs.src/lib/arpinger.sh
 cp HDDcron.sh /opt/USR910x_router/targets/fs.src/lib/
 chmod +x /opt/USR910x_router/targets/fs.src/lib/HDDcron.sh
 cp HDDpowerOn.sh /opt/USR910x_router/targets/fs.src/lib/
 chmod +x /opt/USR910x_router/targets/fs.src/lib/HDDpowerOn.sh
 cp HDDpowerOff.sh /opt/USR910x_router/targets/fs.src/lib/
 chmod +x /opt/USR910x_router/targets/fs.src/lib/HDDpowerOff.sh
 cp HDDmountTest.sh /opt/USR910x_router/targets/fs.src/lib/
 chmod +x /opt/USR910x_router/targets/fs.src/lib/HDDmountTest.sh
 cp HDDmount /opt/USR910x_router/targets/fs.src/lib/
 patch -d /opt/USR910x_router/userapps/opensource/udhcp < udhcp.patch

#copy crontab
 mkdir -p /opt/USR910x_router/targets/fs.src/etc/
 cp crontab /opt/USR910x_router/targets/fs.src/etc/crontab.root
 cp cron.daily /opt/USR910x_router/targets/fs.src/etc/cron.daily.fix
 mkdir -p /opt/USR910x_router/targets/fs.src/usr/sbin/
 cp $HPATCHDIR/delayedcrond /opt/USR910x_router/targets/fs.src/usr/sbin/
 chmod +x /opt/USR910x_router/targets/fs.src/usr/sbin/delayedcrond 

echo "Type \"cd /opt/USR910x_router\" to change to USR9108-Dir"
echo "Some further config files need you attention:"
echo " userapps/opensource/ez-ipupdate/ez.conf"
echo " "
echo -e "\nIf you edited those files to your needs, you are...."
echo "Ready for make. Type \"make PROFILE=96348GW\" for AnnexA image or"
echo "type \"make PROFILE=96348GWB\" for AnnexB."
echo " (AnnexA for Analog+DSL, AnnexB for ISDN+DSL and T-DSL)"
echo "There are many ways to copy the resulting image to yout host machine."
echo " One is to start sshd at the host and use"
echo " scp images/[imagename] [username]@10.0.2.2:"
echo "in qemu standard config 10.0.2.2 should be the host, if this doesn't collide with your network"
